import 'package:b2b_graphql_package/GraphQLConfiguration.dart';
import 'package:b2b_graphql_package/modules/account/user_address_model.dart';
import 'package:b2b_graphql_package/modules/account/user_repo.dart';
import 'package:b2b_graphql_package/modules/checkout/checkout_repo.dart';
import 'package:b2b_graphql_package/modules/payment/payment_repo.dart';
import 'package:b2b_graphql_package/modules/single_pack/single_pack_repo.dart';
import 'package:b2b_graphql_package/modules/wishlist/wishlist_repo.dart';

import 'package:flutter/material.dart';
import 'package:graphql_flutter/graphql_flutter.dart';

import 'modules/account/user_model.dart';
import 'modules/auth/auth_repo.dart';
import 'modules/cart/cart_model.dart';
import 'modules/cart/cart_repo.dart';
import 'modules/local_cart/local_cart.repo.dart';
import 'modules/order/order_repo.dart';
import 'modules/menu/main_menu_repo.dart';

void main() async {
  GraphQLConfiguration graphQLConfiguration = await setupGraphQl();
  runApp(MyApp(graphQLConfiguration.client));
}

Future<GraphQLConfiguration> setupGraphQl() async {
  const serviceUri =
      'https://o6uj5ktn85.execute-api.ap-south-1.amazonaws.com/qa/';
  GraphQLConfiguration graphQLConfiguration =
      GraphQLConfiguration.config(serviceUri);
  await graphQLConfiguration.initiateGraphQl();
  return graphQLConfiguration;
}

class MyApp extends StatelessWidget {
  ValueNotifier<GraphQLClient>? _graphQlClient;
  MyApp(ValueNotifier<GraphQLClient> valueNotifierclient) {
    _graphQlClient = valueNotifierclient;
  }
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return GraphQLProvider(
      client: _graphQlClient,
      child: MaterialApp(
        title: 'Flutter Demo',
        theme: ThemeData(
          primarySwatch: Colors.blue,
        ),
        home: MyHomePage(title: 'Flutter Demo Home Page'),
      ),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key? key, required this.title}) : super(key: key);

  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  int _counter = 0;

  void _incrementCounter() {
    // setState(() {
    //   _counter++;
    // });
    getSettings();
  }

  Future getSettings() async {
    CheckoutRepo check = CheckoutRepo();
    var checkResult = await check.getCartForBooking("616c58eccaacc00009e8fa9f");

    print(checkResult);

    var orderRequest = {
      "order": {
        "billingAddress": {
          "contactName": "Diwahar",
          "companyName": "Rinteger",
          "emailId": "ripsildeveloper4@gmail.com",
          "mobileNumber": 9898989898,
          "address": "street, address",
          "city": "Salem",
          "state": "Tamil Nadu",
          "pincode": 636006,
          "country": "India"
        },
        "shippingAddress": {
          "contactName": "Diwahar",
          "companyName": "Rinteger",
          "emailId": "ripsildeveloper4@gmail.com",
          "mobileNumber": 9898989898,
          "address": "street, address",
          "city": "Salem",
          "state": "Tamil Nadu",
          "pincode": 636006,
          "country": "India"
        },
        "shippingFees": 0,
        "shippingIGST": 0,
        "shippingSGST": 0,
        "shippingCGST": 0,
        "paymentMode": "prepaid",
        "paymentStatus": 1,
        "user": {
          "companyName": "Rinteger",
          "contactName": "Diwahar",
          "mobileNumber": 6382259196,
          "altMobileNumber": 6383780816,
          "emailId": "ripsildeveloper4@gmail.com",
          "gstNumber": "123123123123123",
          "userId": "616c58eccaacc00009e8fa9f"
        },
        "subTotal": 376,
        "totalAmount": 385.4,
        "codCharges": 0,
        "totalGst": 9.4,
        "orderItem": [
          {
            "cgst": 4.7,
            "igst": 9.4,
            "sgst": 4.7,
            "piece": 2,
            "price": 376,
            "quantity": 1,
            "variant": "Size",
            "variantValue": "L",
            "pack": {
              "moq": 1,
              "packName": "Lower Pencil Fit",
              "packVariant": "Size",
              "packId": "616c2b45dc358100080b2b25",
              "packImage": "4M6A0017.JPG",
              "packImageLink":
                  "https://b2b-subscribe.s3.ap-south-1.amazonaws.com/b2b/vilan/product/78-102",
              "packSKU": "PSKU001",
              "categoryId": [
                "616c145672b55700083eecfd",
                "616c148372b55700083eed00",
                "616c14c872b55700083eed03"
              ]
            },
            "set": {
              "setCode": "SET001",
              "setId": "616c2b0fdc358100080b2b15",
              "sku": [
                {
                  "setSkuPrice": 188,
                  "setQuantity": 1,
                  "skuId": "616c2af4dc358100080b2b0b",
                  "sku": "SKU002",
                  "styleCode": "STYLE1",
                  "variationType": "Size",
                  "b2cSellingPrice": 198,
                  "b2bSellingPrice": 188,
                  "mrp": 200,
                  "discount": 0,
                  "productName": "Lower Style fit",
                  "INTsku": "9LB-NT4-NXKH",
                  "sizeAttributeSettingId": "616c189172b55700083eed18",
                  "sizeAttributeSettingFieldValueId":
                      "616c18b972b55700083eed1b",
                  "colorAttributeSettingId": "616c19aa72b55700083eed23",
                  "colorAttributeSettingFieldValueId":
                      "616c1ade72b55700083eed27",
                  "gstTypeId": "616c155a72b55700083eed09",
                  "sizeAttributeName": "size",
                  "colorAttributeName": "color",
                  "sizeAttributeFieldValue": "L",
                  "colorAttributeFieldValue": "Black",
                  "gstValue": 5
                },
                {
                  "setSkuPrice": 188,
                  "setQuantity": 1,
                  "skuId": "616c29f5b480410008cf4ab8",
                  "sku": "SKU001",
                  "styleCode": "STYLE1",
                  "variationType": "Size",
                  "b2cSellingPrice": 198,
                  "b2bSellingPrice": 188,
                  "mrp": 200,
                  "discount": 0,
                  "productName": "Lower Pencil Fit",
                  "INTsku": "PIY-11I-VNOI",
                  "sizeAttributeSettingId": "616c189172b55700083eed18",
                  "sizeAttributeSettingFieldValueId":
                      "616c18b972b55700083eed1b",
                  "colorAttributeSettingId": "616c19aa72b55700083eed23",
                  "colorAttributeSettingFieldValueId":
                      "616c1ade72b55700083eed27",
                  "gstTypeId": "616c155a72b55700083eed09",
                  "sizeAttributeName": "size",
                  "colorAttributeName": "color",
                  "sizeAttributeFieldValue": "L",
                  "colorAttributeFieldValue": "Black",
                  "gstValue": 5
                }
              ]
            }
          }
        ]
      }
    };

    // OrderRepo order = new OrderRepo();
    // var orderReqult = await order.createOrder(orderRequest);

    PaymentRepo pay = new PaymentRepo();
    var result = await pay.createRazorPayOrder(
        "Razorpay", "61cbcc9fad62e00008bd5893", 385.4);

    print(result);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        // Here we take the value from the MyHomePage object that was created by
        // the App.build method, and use it to set our appbar title.
        title: Text(widget.title),
      ),
      body: Center(
        // Center is a layout widget. It takes a single child and positions it
        // in the middle of the parent.
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Text(
              'You have pushed the button this many times:',
            ),
            Text(
              '$_counter',
              style: Theme.of(context).textTheme.headline4,
            ),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _incrementCounter,
        tooltip: 'Increment',
        child: Icon(Icons.add),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}
