import 'package:graphql_flutter/graphql_flutter.dart';

import '../../GraphQLConfiguration.dart';
import 'cart_model.dart';

class CartRepo {
  var graphQLConfiguration;

  var serverCartCount;
  var cart = [];
  var removeCartDetail;
  var updatedCart,
      addedCart,
      delhiveryFees,
      checkCODEnable,
      gateWayEnabled,
      cartSkuWeight,
      delhiveryServiceabilityResult;
  CartRepo() {
    graphQLConfiguration = GraphQLConfiguration();
  }

  Future getServerCartCount(String userId) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var cartCountGql = """ 
              query serverCartCount(\$userId: String) {
              serverCartCount(userId: \$userId)
            }
            """;

    final vble = {
      "userId": userId,
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(cartCountGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var serverCartCountModel = result.data!["serverCartCount"];

      if (serverCartCountModel != null) {
        serverCartCount = cartCountFromJson(serverCartCountModel);
        return serverCartCount;
      }
    }
    return serverCartCount;
  }

  Future getCart(String userId) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var cartGql = """ 
             query getCart(\$userId: String) {
                  getCart(userId: \$userId) {
                    _id
                    packName
                    packId
                    packImage
                    packSku
                    setId
                    quantity
                    piece
                    setPrice
                    price
                    variant
                    igst
                    cgst
                    sgst
                    variantValue
                    isOutofstock
                    maximumQuantity
                    __typename
                  }
                }
            """;

    final vble = {
      "userId": userId,
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(cartGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var serverCartModel = result.data!["getCart"];

      if (serverCartModel != null) {
        cart = cartListFromJson(serverCartModel);
        return cart;
      }
    }
    return cart;
  }

  Future<bool> removeCart(String userId, String cartId) async {
    String cartGql = """mutation removeCart(\$id: String, \$userId: String) {
          removeCart(id: \$id, userId: \$userId)
        }""";
    GraphQLConfiguration graphQLConfiguration = GraphQLConfiguration();
    GraphQLClient _client = graphQLConfiguration.clientToQuery();
    final vble = {"userId": userId, "id": cartId};
    QueryResult result = await _client.mutate(
      MutationOptions(
        document: gql(cartGql),
        variables: vble,
      ),
    );
    if (!result.hasException) {
      var removeCart = result.data!['removeCart'];
      if (removeCart != null) {
        removeCartDetail = removeCart;
      }
    }
    return removeCartDetail;
  }

  Future<bool> updateWishlist(String userId, String packId) async {
    String wishlistGql =
        """mutation changeWishlist(\$userId: String, \$packId: String) {
              changeWishlist(userId: \$userId, packId: \$packId) {
                _id
                __typename
              }
            }""";
    GraphQLConfiguration graphQLConfiguration = GraphQLConfiguration();
    GraphQLClient _client = graphQLConfiguration.clientToQuery();
    final vble = {"packId": packId, "userId": userId};
    QueryResult result = await _client.mutate(
      MutationOptions(
        document: gql(wishlistGql),
        variables: vble,
      ),
    );
    if (!result.hasException) {
      var wishlist = result.data!['changeWishlist'];
      if (wishlist != null && wishlist["_id"] != null) {
        return true;
      } else {
        return true;
      }
    }
    return false;
  }

  Future updateCart(UpdateCartRequest updateCartRequest) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var updateCartGql = """ 
            mutation updateCart(\$cart: CartInput, \$id: String) {
              updateCart(cart: \$cart, id: \$id) {
                _id
                packName
                packId
                packImage
                packSku
                setId
                quantity
                piece
                setPrice
                price
                variant
                igst
                cgst
                sgst
                variantValue
                isOutofstock
                maximumQuantity
                __typename
              }
            }

            """;

    final vble = {"cart": updateCartRequest.cart, "id": updateCartRequest.id};
    QueryResult result = await _client.query(QueryOptions(
      document: gql(updateCartGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var cartUpdate = result.data!['updateCart'];

      if (cartUpdate != null && cartUpdate.length > 0) {
        updatedCart = cartFromJson(cartUpdate);
        return updatedCart;
      }
    }
    return updatedCart;
  }

  Future addCart(AddCartRequest addCartRequest) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var addCartGql = """ 
            mutation addCart(\$cart: CartInput) {
                addCart(cart: \$cart)
              }
            """;

    final vble = {"cart": addCartRequest};
    QueryResult result = await _client.query(QueryOptions(
      document: gql(addCartGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var cartAdd = result.data!['addCart'];

      if (cartAdd != null) {
        addedCart = cartAdd;
        return addedCart;
      }
    }
    return addedCart;
  }

  Future getDelhiveryShippingFess(
      int deliveryPincode, String providerName, double weight) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var shippingFeesGql = """ 
              query getDelhiveryShippingFees(
                \$deliveryPincode: Float
                \$weight: Float
                \$providerName: String
              ) {
                getDelhiveryShippingFees(
                  deliveryPincode: \$deliveryPincode
                  weight: \$weight
                  providerName: \$providerName
                ) {
                  fees
                  shippingIGST
                  shippingSGST
                  shippingCGST
                  __typename
                }
              }

            """;

    final vble = {
      "deliveryPincode": deliveryPincode,
      "weight": weight,
      "providerName": providerName
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(shippingFeesGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var shippingFee = result.data!["getDelhiveryShippingFees"];

      if (shippingFee != null) {
        delhiveryFees = delhiveryShippingFeesFromJson(shippingFee);
        return delhiveryFees;
      }
    }
    return delhiveryFees;
  }

  Future checkCODForCheckout(String userId) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var checkCODGql = """ 
              query checkCODForCheckout(\$userId: String) {
                checkCODForCheckout(userId: \$userId) {
                  cod
                  charges
                  __typename
                }
              }
            """;

    final vble = {
      "userId": userId,
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(checkCODGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var checkCOD = result.data!["checkCODForCheckout"];

      if (checkCOD != null) {
        checkCODEnable = checkCODForCheckoutFromJson(checkCOD);
        return checkCODEnable;
      }
    }
    return checkCODEnable;
  }

  Future getEnabledcheckoutOptions(String gateway) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var gatewayGql = """ 
              query getEnabledcheckoutOptions(\$gateway: String) {
                getEnabledcheckoutOptions(gateway: \$gateway) {
                  _id
                  gatewayName
                  type
                  code
                  description
                  enable
                  list {
                    _id
                    name
                    code
                    logo
                    isPreview
                    __typename
                  }
                  __typename
                }
              }

            """;

    final vble = {
      "gateway": gateway,
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(gatewayGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var gatewayEnable = result.data!["getEnabledcheckoutOptions"];

      if (gatewayEnable != null) {
        gateWayEnabled = enabledcheckoutOptionsFromJson(gatewayEnable);
        return gateWayEnabled;
      }
    }
    return gateWayEnabled;
  }

  Future getCartSkuWeight(String userId) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var skuWeightGql = """ 
           query getCartSkuWeight(\$userId: String) {
              getCartSkuWeight(userId: \$userId)
            }
            """;

    final vble = {
      "userId": userId,
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(skuWeightGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var skuWight = result.data!["getCartSkuWeight"];

      if (skuWight != null) {
        cartSkuWeight = skuWight;
        return cartSkuWeight;
      }
    }
    return cartSkuWeight;
  }

  Future getDelhiveryServiceability(
      int deliveryPincode, String providerName) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var delhiveryServiceabilityGql = """ 
           query getDelhiveryServiceability(
              \$deliveryPincode: Float
              \$providerName: String
            ) {
              getDelhiveryServiceability(
                deliveryPincode: \$deliveryPincode
                providerName: \$providerName
              ) {
                pre_paid
                cash
                pickup
                cod
                estimate
                __typename
              }
            }
            """;

    final vble = {
      "deliveryPincode": deliveryPincode,
      "providerName": providerName
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(delhiveryServiceabilityGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var delhiveryServiceability = result.data!["getDelhiveryServiceability"];

      if (delhiveryServiceability != null) {
        delhiveryServiceabilityResult =
            delhiveryServiceabilityFromJson(delhiveryServiceability);
        return delhiveryServiceabilityResult;
      }
    }
    return delhiveryServiceabilityResult;
  }
}
