import 'package:graphql_flutter/graphql_flutter.dart';

import '../../GraphQLConfiguration.dart';
import 'order_List_model.dart';
import './order_single_model.dart';

class OrderRepo {
  var graphQLConfiguration;

  var orderList;
  var singleOrder;
  var checkCancellation;
  var checkReturn;
  var orderCreated;
  OrderRepo() {
    graphQLConfiguration = GraphQLConfiguration();
  }

  Future getOrderList(String userId, int pageNo, int limit) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var orderListGql = """ 
             query getUserOrder(\$userId: String, \$pageNo: Int, \$limit: Int) {
            getUserOrder(userId: \$userId, pageNo: \$pageNo, limit: \$limit) {
              order {
                orderId
                orderDate
                orderStatus
                totalAmount
                shippingFees
                _id
                __typename
              }
              currentPage
              totalPages
              count
              __typename
            }
          }
            """;

    final vble = {"userId": userId, "pageNo": pageNo, "limit": limit};
    QueryResult result = await _client.query(QueryOptions(
      document: gql(orderListGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var orderListModel = result.data!['getUserOrder'];

      if (orderListModel != null && orderListModel.length > 0) {
        orderList = orderListFromJson(orderListModel);
        return orderList;
      }
    }
    return orderList;
  }

  Future getSingleOrder(String id) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var orderGql = """ 
             query getSingleOrder(\$id: String) {
              getSingleOrder(id: \$id) {
                _id
                orderDate
                orderId
                bookingId
                shippingFees
                paymentStatus
                billingAddress {
                  contactName
                  companyName
                  emailId
                  mobileNumber
                  address
                  city
                  state
                  pincode
                  __typename
                }
                shippingAddress {
                  contactName
                  companyName
                  emailId
                  mobileNumber
                  address
                  city
                  state
                  pincode
                  __typename
                }
                user {
                  contactName
                  mobileNumber
                  emailId
                  userId
                  __typename
                }
                subTotal
                totalGst
                totalAmount
                orderStatus
                orderItem {
                  quantity
                  comments
                  variant
                  variantValue
                  price
                  piece
                  igst
                  cgst
                  sgst
                  pack {
                    categoryId
                    packSKU
                    packName
                    moq
                    packVariant
                    packImage
                    packId
                    __typename
                  }
                  set {
                    setCode
                    setId
                    sku {
                      setSkuPrice
                      setQuantity
                      skuId
                      sku
                      styleCode
                      variationType
                      b2cSellingPrice
                      b2bSellingPrice
                      mrp
                      discount
                      productName
                      INTsku
                      sizeAttributeSettingId
                      sizeAttributeSettingFieldValueId
                      colorAttributeSettingId
                      colorAttributeSettingFieldValueId
                      sizeAttributeName
                      colorAttributeName
                      sizeAttributeFieldValue
                      colorAttributeFieldValue
                      gstTypeId
                      gstValue
                      __typename
                    }
                    __typename
                  }
                  __typename
                }
                __typename
              }
            }

            """;

    final vble = {
      "id": id,
    };
    QueryResult result = await _client.query(QueryOptions(
      document: gql(orderGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var singleOrderModel = result.data!['getSingleOrder'];

      if (singleOrderModel != null && singleOrderModel.length > 0) {
        singleOrder = singleOrderFromJson(singleOrderModel);
        return singleOrder;
      }
    }
    return singleOrder;
  }

  Future checkOrderCancellation(String orderId) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var orderCancellationGql = """ 
                         query checkOrderCancellation(\$orderId: String) {
                  checkOrderCancellation(orderId: \$orderId) {
                    enableCancellation
                    message
                    timePeriod
                    hours
                    __typename
                  }
                }
            """;

    final vble = {"orderId": orderId};
    QueryResult result = await _client.query(QueryOptions(
      document: gql(orderCancellationGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var orderCancel = result.data!['checkOrderCancellation'];

      if (orderCancel != null) {
        checkCancellation = orderCancellationFromJson(orderCancel);
        return checkCancellation;
      }
    }
    return checkCancellation;
  }

  Future checkOrderReturnable(String orderId) async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var orderReturnableGql = """ 
                   query checkOrderReturnable(\$orderId: String) {
                  checkOrderReturnable(orderId: \$orderId) {
                    enableReturn
                    message
                    timePeriod
                    hours
                    __typename
                  }
                }
            """;

    final vble = {"orderId": orderId};
    QueryResult result = await _client.query(QueryOptions(
      document: gql(orderReturnableGql),
      variables: vble,
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var orderRetun = result.data!['checkOrderReturnable'];

      if (orderRetun != null) {
        checkReturn = orderRetunFromJson(orderRetun);
        return checkReturn;
      }
    }
    return checkReturn;
  }

  Future getCancelContent() async {
    GraphQLClient _client = graphQLConfiguration.clientToQuery();

    var orderCancellationGql = """ 
                  query getCancelContent {
                      getCancelContent {
                        reasons {
                          content
                          __typename
                        }
                        message {
                          content
                          __typename
                        }
                        __typename
                      }
                    }
            """;

    QueryResult result = await _client.query(QueryOptions(
      document: gql(orderCancellationGql),
      variables: {},
      fetchPolicy: FetchPolicy.networkOnly,
    ));

    if (!result.hasException) {
      var orderCancelContent = result.data!['getCancelContent'];

      if (orderCancelContent != null) {
        checkCancellation = orderCancelContentFromJson(orderCancelContent);
        return checkCancellation;
      }
    }
    return checkCancellation;
  }

  Future<bool> cancelOrder(String reason, String orderId) async {
    String addUserGql =
        """mutation cancelOrder(\$reason: String, \$orderId: String) {
            cancelOrder(reason: \$reason, orderId: \$orderId) {
              _id
              orderDate
              orderId
              bookingId
              cancellationReason
              cancellationDate
              returnReason
              returnRequestedDate
              returnRejectionReason
              shippingFees
              paymentStatus
              paymentMode
              codCharges
              history {
                action
                type
                customerView
                date
                __typename
              }
              billingAddress {
                contactName
                companyName
                emailId
                mobileNumber
                address
                city
                state
                pincode
                __typename
              }
              shippingAddress {
                contactName
                companyName
                emailId
                mobileNumber
                address
                city
                state
                pincode
                __typename
              }
              user {
                contactName
                mobileNumber
                emailId
                userId
                __typename
              }
              subTotal
              totalGst
              totalAmount
              orderStatus
              orderItem {
                quantity
                comments
                variant
                variantValue
                price
                piece
                igst
                cgst
                sgst
                pack {
                  categoryId
                  packSKU
                  packName
                  moq
                  packVariant
                  packImage
                  packId
                  __typename
                }
                set {
                  setCode
                  setId
                  sku {
                    setSkuPrice
                    setQuantity
                    skuId
                    sku
                    styleCode
                    variationType
                    b2cSellingPrice
                    b2bSellingPrice
                    mrp
                    discount
                    productName
                    INTsku
                    sizeAttributeSettingId
                    sizeAttributeSettingFieldValueId
                    colorAttributeSettingId
                    colorAttributeSettingFieldValueId
                    sizeAttributeName
                    colorAttributeName
                    sizeAttributeFieldValue
                    colorAttributeFieldValue
                    gstTypeId
                    gstValue
                    __typename
                  }
                  __typename
                }
                __typename
              }
              __typename
            }
          }
          """;
    GraphQLConfiguration graphQLConfiguration = GraphQLConfiguration();
    GraphQLClient _client = graphQLConfiguration.clientToQuery();
    final vble = {"reason": reason, "orderId": orderId};
    QueryResult result = await _client.mutate(
      MutationOptions(
        document: gql(addUserGql),
        variables: vble,
      ),
    );
    if (!result.hasException) {
      var cancelOrder = result.data!['cancelOrder'];
      if (cancelOrder != null && cancelOrder["_id"] != null) {
        return true;
      } else {
        return true;
      }
    }
    return false;
  }

  Future<bool> createOrder(CreateOrderRequest order) async {
    String createOrderGql = """mutation createOrder(\$order: order) {
          createOrder(order: \$order) {
            _id
            shippingFees
            totalAmount
            _id
            orderDate
            orderId
            bookingId
            shippingFees
            history {
              action
              type
              customerView
              date
              __typename
            }
            paymentStatus
            billingAddress {
              contactName
              companyName
              emailId
              mobileNumber
              address
              city
              state
              pincode
              __typename
            }
            shippingAddress {
              contactName
              companyName
              emailId
              mobileNumber
              address
              city
              state
              pincode
              __typename
            }
            user {
              contactName
              mobileNumber
              emailId
              userId
              __typename
            }
            subTotal
            totalGst
            totalAmount
            orderStatus
            orderItem {
              quantity
              comments
              variant
              variantValue
              price
              piece
              igst
              cgst
              sgst
              pack {
                categoryId
                packSKU
                packName
                moq
                packVariant
                packImage
                packId
                __typename
              }
              set {
                setCode
                setId
                sku {
                  setSkuPrice
                  setQuantity
                  skuId
                  sku
                  styleCode
                  variationType
                  b2cSellingPrice
                  b2bSellingPrice
                  mrp
                  discount
                  productName
                  INTsku
                  sizeAttributeSettingId
                  sizeAttributeSettingFieldValueId
                  colorAttributeSettingId
                  colorAttributeSettingFieldValueId
                  sizeAttributeName
                  colorAttributeName
                  sizeAttributeFieldValue
                  colorAttributeFieldValue
                  gstTypeId
                  gstValue
                  __typename
                }
                __typename
              }
              __typename
            }
            __typename
          }
        }

          """;
    GraphQLConfiguration graphQLConfiguration = GraphQLConfiguration();
    GraphQLClient _client = graphQLConfiguration.clientToQuery();
    final vble = {"order": order};
    QueryResult result = await _client.mutate(
      MutationOptions(
        document: gql(createOrderGql),
        variables: vble,
      ),
    );
    if (!result.hasException) {
      var createOrder = result.data!['createOrder'];
      if (createOrder != null && createOrder["_id"] != null) {
        orderCreated = createOrderResponseFromJson(createOrder);
      }
    }
    return orderCreated;
  }
}
